/**
`kano-alert`

Example:
    <kano-alert heading=""
                       text=""
                       icon-color="">
        <div slot="icon"></div>
        <div slot="actions"></div>
    </kano-alert>

### Slotted content
The default icon, as well as the action controls can be replaced with slotted content.
You can use 'kano-alert-primary' and 'kano-alert-secondary' classes for buttons in the actions slot.

```html

<kano-alert>
    <button class="kano-alert-primary" slot="actions">Confirm</button>
    <button class="kano-alert-secondary" slot="actions">Cancel</button>
</kano-alert>

```

The following custom properties and mixins are also available for styling:

Custom property | Description | Default
----------------|-------------|----------
`--kano-alert`                  | Mixin applied to kano-alert    | `{}`
`--kano-alert-icon-background`  | Background of the icon                | `#8d8d8d`
`--kano-alert-header`           | Mixin applied to the header           | `{}`
`--kano-alert-text`             | Mixin applied to the text             | `{}`
`--kano-alert-action-container` | Mixin applied to the action container | `{}`

@group Kano Elements
@hero hero.svg
@demo ./kano-alert/demo/kano-alert.html
*/
import { PaperDialogBehavior } from '@polymer/paper-dialog-behavior/paper-dialog-behavior.js';
import { PolymerElement } from '@polymer/polymer/polymer-element.js';
import 'marked/lib/marked.js';
import '@kano/styles/typography.js';
declare const KanoAlert_base: new () => PolymerElement & PaperDialogBehavior;
export declare class KanoAlert extends KanoAlert_base {
    static readonly template: HTMLTemplateElement;
    heading: string;
    text: string;
    markedText: string;
    _computedMarked(src: string): string;
    open(): void;
    close(): void;
}
export {};
