/**
 * \file pc-channel.js
 *
 * \copyright Copyright (C) 2019-2020 Kano Computing Ltd.
 *
 *  Implementation web channel interface for OS
 */

import { sendUserDataIabIOS, quitIabIOS } from './ios.js';
// QWebChannel main window
let profile = null;
let mainWindow = null;

/**
 * Check if QWebChannel is available and expose
 * `profileComponents` as `mainWindow`
 */

const configureQWebChannel = () => {
    if (profile && mainWindow) {
       return;
    }

    try {
        new QWebChannel(qt.webChannelTransport, function (channel) {
            mainWindow = channel.objects.appWindowComponent;
            profile = channel.objects.profileComponent;
        });
    } catch (error) {
        console.log('Problem with QWebChannel', error);
    }
}

/**
 * QWebChannel calls
 */

export const sendUserData = (id, username, email, token, duration, consent, method) => {
    return new Promise(function(resolve, reject) {
        try {
            configureQWebChannel();
            if (profile) {
                profile.setUserData(id, username, email, token, duration, consent);
            }
            // Send a message to the parent window. Kash can use that when it is used in an iframe or an In-App Browser
            window.parent.postMessage({ m: 'auth-callback', a: { id, username, email, token, method } }, '*');
            sendUserDataIabIOS(id, username, email, token, method);
            resolve();
        } catch(e) {
            console.error("Could not send user data through OS channel", e);
            reject(e);
        }
    });
};

export const quit = () => {
    configureQWebChannel();
    // A small delay to keep the success screen visible for the user
    if (mainWindow) {
        mainWindow.close();
    }
    // Send a message to the parent window. Kash can use that when it is used in an iframe or an In-App Browser
    window.parent.postMessage({ m: 'quit' }, '*');
    quitIabIOS();
};

export const notifyViewChange = (view) => {
    configureQWebChannel();
    window.parent.postMessage({ m: 'change-view', a: view }, '*');
};
